Object.defineProperty(exports, "__esModule", { value: true });
const model_validation_error_report_1 = require("./model_validation_error_report");
const validates_method_1 = require("./validates_method");
/** バリデーションを記述する関数に渡されるvalidatesとvalidate */
class ValidationMethods {
    /**
     * コンストラクタ
     * @param model 適用するモデル
     * @param validation バリデーション情報を管理するオブジェクト
     */
    constructor(model, validation) {
        this.model = model.eager();
        this.errors = new model_validation_error_report_1.ModelValidationErrorReports(model, validation);
        this.validates = this.validates.bind(this);
        this.validate = this.validate.bind(this);
    }
    /**
     * 汎用のバリデーターでバリデーションする
     *
     * ActiveModel::Validations::ClassMethods#validates相当
     * @param column カラム
     * @param method バリデーション方法
     * @param params バリデーション方法による引数
     * @param condition ifあるいはunlessによる絞り込み
     */
    // tslint:disable-next-line no-shadowed-variable
    validates(column, method, params, condition = {}) {
        const model = this.filterModelByCondition(condition);
        switch (method) {
            case "presence":
                return validates_method_1.ValidatesMethod.Method.presence(this.errors, params, model, column);
            case "presenceBelongsTo":
                return validates_method_1.ValidatesMethod.Method.presenceBelongsTo(this.errors, params, model, column);
            case "uniqueness":
                return validates_method_1.ValidatesMethod.Method.uniqueness(this.errors, params, model, column);
            case "length":
                return validates_method_1.ValidatesMethod.Method.length(this.errors, params, model, column);
            case "format":
                return validates_method_1.ValidatesMethod.Method.format(this.errors, params, model, column);
            case "numericality":
                return validates_method_1.ValidatesMethod.Method.numericality(this.errors, params, model, column);
            case "inclusion":
                return validates_method_1.ValidatesMethod.Method.inclusion(this.errors, params, model, column);
            case "exclusion":
                return validates_method_1.ValidatesMethod.Method.exclusion(this.errors, params, model, column);
            default:
                throw new Error(`${method} is not validates method`);
        }
    }
    /**
     * カスタムのバリデーション方法でバリデートする
     *
     * ActiveModel::Validations::ClassMethods#validate(ブロック渡し)相当
     * @param validator カスタムバリデーター
     * @param condition ifあるいはunlessによる絞り込み
     */
    validate(validator, condition = {}) {
        const model = this.filterModelByCondition(condition);
        validator(this.errors, model);
    }
    filterModelByCondition(condition) {
        let model = this.model;
        if (condition.if)
            model = model.filter(condition.if);
        if (condition.unless)
            model = model.filter((record) => !condition.unless(record));
        return model;
    }
}
exports.ValidationMethods = ValidationMethods;
//# sourceMappingURL=validation_methods.js.map